@@ -28,14 +28,25 @@ class ActionModule(object):
     def __init__(self, runner):
         self.runner = runner
 
-    def run(self, conn, tmp, module_name, module_args, inject):
-        args = utils.parse_kv(module_args)
+    def run(self, conn, tmp, module_name, module_args, inject, complex_args=None, **kwargs):
+        args = {}
+        if complex_args:
+            args.update(complex_args)
+
+        # attempt to prevent confusing messages when the variable didn't interpolate
+        module_args = module_args.replace("{{ ","{{").replace(" }}","}}")
+
+        kv = utils.parse_kv(module_args)
+        args.update(kv)
         if not 'msg' in args:
             args['msg'] = 'Hello world!'
 
         if 'fail' in args and utils.boolean(args['fail']):
             result = dict(failed=True, msg=args['msg'])
         else:
-            result = dict(msg=str(args['msg']))
+            result = dict(msg=args['msg'])
+
+        # force flag to make debug output module always verbose
+        result['verbose_always'] = True
 
         return ReturnData(conn=conn, result=result)
